require 'csv'

module TabTools

  class Anonymizer

    attr_reader   :anonWords, :low, :high
    attr_accessor :input, :output, :field

    @minWords = 2
    @maxWords = 4

    @@anonymousText = <<ANONTEXT
    Lorem ipsum dolor sit amet consectetur adipiscing elit Mauris urna erat volutpat a ullamcorper sit amet 
    convallis quis dui Ut sem erat euismod in iaculis id tincidunt aliquam magna Vivamus lobortis eu erat in feugiat 
    Vestibulum fermentum dictum fermentum Class aptent taciti sociosqu ad litora torquent per conubia nostra 
    per inceptos himenaeos Ut quis rutrum mi Mauris sit amet cursus nisi Suspendisse accumsan leo ac lobortis egestas 
    sapien eros mattis lorem eu posuere mi justo in nunc Sed felis ligula egestas vel nulla sed egestas lobortis justo
    Donec quis interdum velit Proin felis arcu vestibulum non augue nec ultricies pulvinar velit Nunc ac libero velit 
    Aenean bibendum varius nisi ut molestie Nam tempus nec lacus eget malesuada Donec et accumsan dolor et ullamcorper ante 
    Etiam quis consectetur leo Suspendisse mattis sagittis imperdiet Phasellus facilisis eget justo ac mollis 
    Phasellus eleifend tortor est sit amet venenatis enim fringilla id Vestibulum rutrum tempus tortor ac volutpat 
    Nullam non imperdiet lorem Cras tincidunt porttitor condimentum Pellentesque tincidunt magna id aliquet interdum augue 
    nisl convallis enim volutpat mollis urna metus et lacus Donec tortor sapien ultrices sit amet blandit quis gravida ut 
    massa Mauris mauris lacus pretium sit amet risus quis blandit viverra lorem Nunc fringilla elit a elementum gravida 
    Duis vel sodales elit eget tempor leo Quisque in semper nisl Fusce eget risus sollicitudin molestie diam et tincidunt 
    sem Curabitur accumsan pretium interdum Suspendisse semper enim ac augue hendrerit vel sollicitudin diam mollis 
    Curabitur lacinia ante nec eros maximus a vehicula velit aliquet In ullamcorper nec ante eu dictum Proin mi enim 
    malesuada ut ornare sit amet maximus commodo ipsum Nullam venenatis purus non sollicitudin sollicitudin augue 
    augue euismod lorem ut hendrerit velit neque sed nibh Class aptent taciti sociosqu ad litora torquent per conubia 
    nostra per inceptos himenaeos Proin ac elementum nisl
ANONTEXT

    @codedWords = {}

    def initialize
      @anonWords = @@anonymousText.gsub(/\n/,' ').split
      @coded = {}
      @low   = 0
      @limit = @anonWords.length-1
      @rand  = Random.new
      @minWords = 2
      @maxWords = 4
    end

    def anonymize(input, fields,  output)
      puts "Anonymizing %-25s  to  %-25s  - %s " % [input, output, fields.inspect]
      raise 'The input file must be specified' if input.nil?
      raise 'The input file must exist'        unless File.exist?(input)
      raise 'The field(s) to anonymize must be provided as a String or an array of Strings' unless fields.instance_of?(String) || fields.instance_of?(Array)
      @input    = input
      @output   = output
      @anonVals = {}
      csvFields = CSV.open(@input, &:readline) 
      output    = CSV.open(@output,'w')
      output << csvFields
      fields = [fields] if fields.is_a? String
      CSV.foreach('input.csv', :headers => true) do |row|
        fields.each do |field|
          row[field] = anonWords(row[field]) unless row[field].nil?
        end
        output << row
      end
    end

    def anonWords value
      return @anonVals[value] unless @anonVals[value].nil?
      range = rand(@minWords..@maxWords)
      limit = @limit - range 
      first = rand(range..limit)
      last  = first + range - 1
      words = @anonWords[first..last].join(' ')
      @anonVals[value] = words
    end

    def codeValue value
      if @codedWords[value].nil?
        @codedWords[value] = anonWords
      end
      @codedWords[value]
    end


  end # class Anonymizer

end # module utility